import QtQuick 1.1
import se.flir 1.0

// Archive
// -------
// The archive. Uses a ThumbnailArchive and a PreviewArchive to show the archive images.

Item{
    // Signals and properties
    signal hideArchive

    // Memory for the archive state.
    property variant lastArchiveView: ArchiveHandler.Thumbnails

    id: root
    width: grid.width
    height: grid.height
    objectName: "archive"

    Rectangle {
        id: archiveBackground
        anchors.fill: parent
        color: colors.background
    }

    // Debug output
//    onHideArchive: console.log("Archive::hideArchive()")

    // Handle big states
    function hide() {
//        console.log("Hiding archive")
        if(archiveHandler.view !== ArchiveHandler.ImageSelector && lastArchiveView !== ArchiveHandler.Thumbnails)
            archiveHandler.view = ArchiveHandler.Thumbnails
        lastArchiveView = archiveHandler.view

        archiveHandler.view = ArchiveHandler.Invisible
    }

    function show() {
//        console.log("Showing archive")
        root.visible = true // Needs this to accept focus
        scope.forceActiveFocus()
        archiveHandler.view = lastArchiveView
        console.log("archiveHandler.view: " + archiveHandler.view)
    }

    FocusScope {
        anchors.fill: parent
        id: scope
        objectName: "archiveFocusScope"

        // Thumbnail archive
        ThumbnailArchive {
            id: thumbnailArchive
            onHideArchive: {
                root.hideArchive()
            }
            onShowPreviewArchive: archiveHandler.view = ArchiveHandler.ImageSelector
            focus: root.state == "thumbnailView"
        }

        // Preview archive
        PreviewArchive {
            id: previewArchive
            onShowThumbnailArchive: archiveHandler.view = ArchiveHandler.Thumbnails
            onShowArchiveToolBar: menus.navigateUp()
            focus: root.state == "imageSelectorView"
        }

        // No images message
        MessageBox {
            id: emptyArchiveNotification
            //: Message header. Displayed when user enters the image archive and there are no saved images on the memory card.
            //% "No images found"
            title: qsTrId("ID_ARCHIVE_NO_IMAGES_HEADER") + translation.update
            //: Message text body. Displayed with message header when no images are found.
            //% "There are no saved images."
            text: qsTrId("ID_ARCHIVE_NO_IMAGES_BODY") + translation.update
            hasButtons: false
            iconStyle: iconEnum.info
            focus: root.state == "archiveIsEmpty"

            // Keyboard handling
            Keys.onReleased: {
                if (event.key === keys.archive || event.key === keys.back) {
                    hideArchive()
                    event.accepted = true
                }
            }
        }

        // Indexing images message
        MessageBox {
            id: indexingNotification
            //: Message header. Displayed when user enters the image archive and the archive is indexing the data.
            //% "Indexing images"
            title: qsTrId("ID_ARCHIVE_INDEXING_IMAGES_HEADER") + translation.update
            //: Message text body. Displayed with message header when the archive is indexing the data.
            //% "Reading image data..."
            text: qsTrId("ID_ARCHIVE_INDEXING_IMAGES_BODY") + translation.update
            hasButtons: false
            iconStyle: iconEnum.info
            focus: root.state == "archive"

            // Keyboard handling
            Keys.onReleased: {
                if (event.key === keys.archive || event.key === keys.back) {
                    hideArchive()
                    event.accepted = true
                }
            }
        }

        Image {
            id: fullscreenImage
            anchors.fill: parent
            sourceSize.width: grid.width
            sourceSize.height: grid.height
            width: grid.width
            height: grid.height
            cache: false
            smooth: true
            asynchronous: true
            focus: root.state == "fullscreen"

            Behavior on opacity {
                NumberAnimation {
                    duration: 300
                    easing.type: Easing.InOutQuad
                }
            }

            Keys.onReleased: {
                if (event.key === keys.back) {
                    archiveHandler.view = ArchiveHandler.ImageSelector
                    event.accepted = true
                }
                else if(event.key === keys.select) {
                    menus.navigateUp()
                    event.accepted = true
                }
            }
        }

        MessageBox {
            id: deleteQuestion

            property string deleteType: "image"

            hasButtons: true
            iconStyle: iconEnum.warning
            //: Imperative. Button. Select to delete image or voice comment
            //% "Delete"
            okButtontext: qsTrId("ID_BUTTON_DELETE") +  translation.update
            //: Imperative. Button. Select to cancel delete action
            //% "Cancel"
            cancelButtontext: qsTrId("ID_BUTTON_CANCEL") +  translation.update
            focus: root.state == "deleteDialog"

            onCancelSelected: {
                archiveHandler.view = ArchiveHandler.ImageSelector
                previewArchive.forceActiveFocus()
            }
            onOkSelected: {
                if (deleteType === "image")
                {
                archiveHandler.removeCurrentImage();
                archiveHandler.view = ArchiveHandler.ImageSelector
                // Really convince qml about the focus for empty archives...
                if (archiveHandler.model.empty)
                    emptyArchiveNotification.forceActiveFocus()
                else
                    previewArchive.forceActiveFocus()
            }
                else
                {
                    archiveHandler.deleteVoiceAnnotation()
                    archiveHandler.view = ArchiveHandler.ImageSelector
                    previewArchive.forceActiveFocus()
        }
            }
        }

        // Message box
        MessageBox {
            id: archiveInfoDiag
            focus: root.state == "infoDialog"
            title: archiveHandler.model.currentGroupFullName
            tableModel: infoMsgBoxModel
            onCancelSelected: {
                archiveHandler.view = ArchiveHandler.ImageSelector
                previewArchive.forceActiveFocus()
            }
        }

        MessageBox {
            id: archiveWarningMessage
            //: Message header displayed when headset is not paired to the camera.
            //% "Headset not found"
            title: qsTrId("ID_HEADSET_NOT_PAIRED_HEADER") + translation.update
            //: Message text displayed when headset is not paired to the camera.
            //% "Please connect a headset or go to Settings to pair with a new headset."
            text: qsTrId("ID_HEADSET_NOT_PAIRED_TEXT") + translation.update
            iconStyle: iconEnum.warning
            focus: root.state == "warningMessage"
            hasButtons: false

            onCancelSelected: {
                root.state = "imageSelectorView"
            }
        }


        // Keyboard handling
        Keys.onPressed: {
            if (event.key === keys.archive) {
                // Prevent this from falling through to desktop
                event.accepted = true
            }
            if (event.key === keys.trigger && event.isAutoRepeat === false) {
                hideArchive()
                event.accepted = true
            }
        }
        Keys.onReleased: {
            if (event.key === keys.archive || event.key === keys.back) {
                hideArchive()
                event.accepted = true
            }
        }
    }

    ListModel {     // This model is populated when opening info dialog
        id: infoMsgBoxModel
    }

    ListModel {     // Populate this before showing it
        id: deleteMsgBoxModel
    }

    Connections { target: menus.menuItem("archive_delete")
        onTriggered: {
            archiveHandler.view = ArchiveHandler.Fullscreen;
            deleteQuestion.deleteType = "image"
            //: Question. Displayed when user selects to delete an image file. %1 is a placeholder for a filename.
            //% "Delete %1?"
            deleteQuestion.title = archiveHandler.model.currentGroupName === "" ? qsTrId("DELETE_X_HEADER") : qsTrId("DELETE_X_HEADER").arg(archiveHandler.model.currentGroupName)
            if (archiveHandler.model.currentGroupIsVideo) {
                //: Information text displayed with question when user selects to delete a video file.
                //% "This will delete the current video."
                deleteQuestion.text = qsTrId("DELETE_X_VIDEO_BODY") +  translation.update
            } else {
                //: Information text displayed with question when user selects to delete an image.
                //% "This will delete the current image."
                deleteQuestion.text = qsTrId("DELETE_X_BODY") +  translation.update
            }
            root.state = "deleteDialog"
        }
    }

    Connections { target: menus.menuItem("archive_voice_annotation_delete")
        onTriggered: {
            archiveHandler.view = ArchiveHandler.Fullscreen;
            deleteQuestion.deleteType = "voice"
            //: Question. Displayed when user selects to delete an voice comment.
            //% "Delete voice annotation?"
            deleteQuestion.title = qsTrId("DELETE_VOICE_HEADER")
            //: Information text displayed with question when user selects to delete a voice comment.
            //% "This will delete the current voice annotation."
            deleteQuestion.text = qsTrId("DELETE_VOICE_BODY") +  translation.update

            root.state = "deleteDialog"
        }
    }


    // Listen to changes in the model and set the corresponding view.
    // Note: this generates an error at startup, as not everything is setup, but it works!
    Connections { target: archiveHandler.model
        onCurrentIndexChanged: {
            if (state == "thumbnailView") {
                if (thumbnailArchive.currentIndex !== archiveHandler.model.currentIndex) {
                    thumbnailArchive.positionViewAtIndex(archiveHandler.model.currentIndex, ListView.Contain)
                    thumbnailArchive.currentIndex = archiveHandler.model.currentIndex
                }
            } else if (state == "imageSelectorView") {
                if (previewArchive.currentIndex !== archiveHandler.model.currentIndex) {
                    previewArchive.positionViewAtIndex(archiveHandler.model.currentIndex, ListView.Contain)
                    previewArchive.currentIndex = archiveHandler.model.currentIndex
                }
            }
        }
        // Invalidate the stored index when re-reading stuff
        onStatusChanged: {
            if (archiveHandler.model.status == ArchiveModel.StatusStarting) {
                thumbnailArchive.currentIndex = -1
            }
        }
    }

    Connections { target: archiveHandler
        onHeadsetNotAvailable: {
            console.log("Missing headset!")
            menus.closeMenu()
            root.state = "warningMessage"
        }
    }

    Connections { target: menus.menuItem("archive_info")
        onTriggered: {
            archiveInfoDiag.hasButtons = false
            archiveInfoDiag.iconStyle = archiveInfoDiag.iconEnum.info
            archiveHandler.view = ArchiveHandler.Fullscreen;
            root.state = "infoDialog"
        }
    }

    Connections {
        target: archiveHandler.model
        onCurrentArchiveDataChanged: {
            infoMsgBoxModel.clear()
            if (archiveHandler.model.currentGroupIRWidth !== 0)
            {
                //: Noun. Emissivity of image.
                //% "Emissivity:"
                infoMsgBoxModel.append({ "label": qsTrId("ID_ARCHIVE_EMISSIVITY"),
                        // 0x3b5 is the unicode code point for epsilon
                        "value": "\u03b5 " + archiveHandler.model.currentGroupEmissivity})
                //: Noun. Reflected temperature of image.
                //% "Reflected temperature:"
                infoMsgBoxModel.append({ "label": qsTrId("ID_ARCHIVE_REFLECTED_TEMPERATURE"), "value": archiveHandler.model.currentGroupReflTemp + " " + qsTrId(greenbox.resultTable.tempUnit)})
                if (archiveInfo.distance)
                {
                    //: Noun. Distance to measured object in saved image
                    //% "Distance:"
                    infoMsgBoxModel.append({ "label": qsTrId("ID_ARCHIVE_OBJECT_DISTANCE"),
                                               "value": qsTrId(greenbox.resultTable.distUnit).arg(archiveHandler.model.currentGroupObjectDistance)})
                }
                if (archiveInfo.relHum)
                {
                    //: Noun. Relative humidity of saved image
                    //% "Relative humidity:"
                    infoMsgBoxModel.append({ "label": qsTrId("ID_ARCHIVE_RELATIVE_HUMIDITY"),
                                               "value": archiveHandler.model.currentGroupRelHumidity + " %"})
                }
                if (archiveInfo.atmTemp)
                {
                    //: Noun. Atmospheric temperature of saved image
                    //% "Atmospheric temperature:"
                    infoMsgBoxModel.append({ "label": qsTrId("ID_ARCHIVE_ATMOSPHERIC_TEMPERATURE"),
                                               "value": archiveHandler.model.currentGroupAtmTemp + " " + qsTrId(greenbox.resultTable.tempUnit)})
                }
                if (archiveInfo.extOptics)
                {
                    if (archiveHandler.model.currentGroupExtOpticsTransm !== "1")
                    {
                        //: Noun. External optics compensation of saved image
                        //% "Window compensation:"
                        infoMsgBoxModel.append({ "label": qsTrId("ID_ARCHIVE_EXTERNAL_OPTICS"),
                                                   "value": archiveHandler.model.currentGroupExtOpticsTransm + "@" + archiveHandler.model.currentGroupExtOpticsTemp + " " + qsTrId(greenbox.resultTable.tempUnit)})
                    }
                }
                if (archiveHandler.model.currentGroupGPSLatitude !== "")
                {
                    //: Noun. GPS - Global positioning system
                    //% "GPS:"
                    infoMsgBoxModel.append({ "label": qsTrId("ID_ARCHIVE_GPS"),
                                               "value": archiveHandler.model.currentGroupGPSLatitude + ", " + archiveHandler.model.currentGroupGPSLongitude})
                }
                if (archiveHandler.model.currentGroupCompass !== "")
                {
                    //: Noun. Compass
                    //% "Compass:"
                    infoMsgBoxModel.append({ "label": qsTrId("ID_ARCHIVE_COMPASS"),
                                               "value": qsTrId(archiveHandler.model.currentGroupCompass)})
                }

                //: Noun. Resoultion of IR image
                //% "Thermal resolution:"
                infoMsgBoxModel.append({ "label": qsTrId("ID_ARCHIVE_IR_RESOLUTION"),
                                          "value": archiveHandler.model.currentGroupIRWidth + "x" + archiveHandler.model.currentGroupIRHeight})
                //: Noun. Resolution of visual image.
                //% "Digital resolution:"
                infoMsgBoxModel.append({ "label": qsTrId("ID_ARCHIVE_VIS_RESOLUTION"),
                                         "value": archiveHandler.model.currentGroupDCWidth + "x" + archiveHandler.model.currentGroupDCHeight})
                //: Noun. Lens (field of view of lens used to take image)
                //% "Lens:"
                infoMsgBoxModel.append({ "label": qsTrId("ID_ARCHIVE_LENS"),
                                          "value": archiveHandler.model.currentGroupFOV + "\u00B0"}) // 0xB0 is the unicode point for degrees.
                //: The date the image was taken
                //% "Date taken:"
                infoMsgBoxModel.append({ "label": qsTrId("ID_ARCHIVE_DATE_TAKEN"),
                                          "value": archiveHandler.model.currentGroupDate})
                //: The time the image was taken
                //% "Time:"
                infoMsgBoxModel.append({ "label": qsTrId("ID_ARCHIVE_TIME_TAKEN"),
                                          "value": archiveHandler.model.currentGroupTime})
            }
            else if (archiveHandler.model.currentGroupIsVideo)
            {
                //: Noun. Resolution of video
                //% "Video resolution:"
                infoMsgBoxModel.append({ "label": qsTrId("ID_ARCHIVE_VIDEO_RESOLUTION"),
                                          "value": archiveHandler.model.currentGroupImgWidth + "x" + archiveHandler.model.currentGroupImgHeight})
            }
            else if (archiveHandler.model.currentGroupImgWidth !== 0)
            {
                //: Noun. Resolution of visual image.
                //% "Digital resolution:"
                infoMsgBoxModel.append({ "label": qsTrId("ID_ARCHIVE_VIS_RESOLUTION"),
                                         "value": archiveHandler.model.currentGroupImgWidth + "x" + archiveHandler.model.currentGroupImgHeight})
            }
            else
            {
                //: While waiting for the information to be fetched.
                //% "Please wait..."
                infoMsgBoxModel.append({ "label": qsTrId("ID_WAITFORIT"), "value": ""})
            }
        }
    }

    Connections { // get keyboard focus from menu to the right thing after it closes.
        target: menus
        onMenuClosed: {
            // Only get the focus if the menu has it
            var toolbars = greenbox.GetNamedObject("globalToolbars")
            if (toolbars !== null && !toolbars.focus)
                return
            if (state == "fullscreen") {
                fullscreenImage.forceActiveFocus()
            } else if (state == "imageSelectorView") {
                previewArchive.forceActiveFocus()
            }
        }
    }

    states: [
        State {
            name: "hidden"
            when: archiveHandler.view == ArchiveHandler.Invisible
            PropertyChanges { target: root; visible: false }
        },
        State {
            name: "visible"
            PropertyChanges { target: root; visible: true }
            PropertyChanges { target: archiveBackground; visible: true }
            PropertyChanges { target: thumbnailArchive; opacity: 0 }
            PropertyChanges { target: previewArchive; visible: false }
            PropertyChanges { target: archiveInfoDiag; opacity: 0 }
            PropertyChanges { target: deleteQuestion; opacity: 0 }
            PropertyChanges { target: archiveWarningMessage; opacity: 0 }
            PropertyChanges { target: fullscreenImage; opacity: 0 }
            PropertyChanges { target: emptyArchiveNotification; opacity: 0 }
            PropertyChanges { target: storageBlockedByMsdNotification; opacity: 0 }
            PropertyChanges { target: missingStorageCardNotification; opacity: 0 }
            PropertyChanges { target: fullscreenImage; source: "" }
//            PropertyChanges { target: videoPlaybackOverlay; visible: false }
            PropertyChanges { target: archiveHandler; videoPlaybackActive: false }
            StateChangeScript { script: console.log("state = visible") }
        },
        State {
            name: "archiveIsEmpty"
            extend: "visible"
            when: archiveHandler.view != ArchiveHandler.Invisible && archiveHandler.model.empty &&
                  archiveHandler.model.status !== ArchiveModel.StatusIndexing
            PropertyChanges { target: emptyArchiveNotification; opacity: 1 }
            StateChangeScript {
                script: {
                    console.log("state = archiveIsEmpty")
                    // Active focus is lost here sometimes. Make sure it stays put.
                    if (scope.focus && !scope.activeFocus)
                        emptyArchiveNotification.forceActiveFocus()
                }
            }
        },
        State {
            name: "archiveIsIndexing"
            extend: "visible"
            when: archiveHandler.view != ArchiveHandler.Invisible && archiveHandler.model.status == ArchiveModel.StatusIndexing
            PropertyChanges { target: indexingNotification; opacity: 1 }
            StateChangeScript { script: console.log("state = archiveIsIndexing") }
        },
        State {
            name: "thumbnailView"
            extend: "visible"
            when: archiveHandler.view == ArchiveHandler.Thumbnails &&
                  !archiveHandler.model.empty &&
                  archiveHandler.model.status === ArchiveModel.StatusReady
            PropertyChanges { target: thumbnailArchive; opacity: 1 }
            StateChangeScript {
                script: {
                    console.log("state = thumbnailView")
                    if (thumbnailArchive.currentIndex !== archiveHandler.model.currentIndex) {
                        thumbnailArchive.positionViewAtIndex(archiveHandler.model.currentIndex, ListView.Contain)
                        thumbnailArchive.currentIndex = archiveHandler.model.currentIndex
                    }
                }
            }
        },
        State {
            name: "imageSelectorView"
            extend: "visible"
            when: archiveHandler.view == ArchiveHandler.ImageSelector &&
                  !archiveHandler.model.empty &&
                  archiveHandler.model.status === ArchiveModel.StatusReady
            PropertyChanges { target: previewArchive; visible: true }
            StateChangeScript {
                script: {
                    console.log("state = imageSelectorView")
                    previewArchive.forceActiveFocus()
                    if (previewArchive.currentIndex !== archiveHandler.model.currentIndex) {
                        previewArchive.positionViewAtIndex(archiveHandler.model.currentIndex, ListView.Contain)
                        previewArchive.currentIndex = archiveHandler.model.currentIndex
                    }
                }
            }
        },
        State {
            name: "fullscreen"
            extend: "visible"
            when: archiveHandler.view == ArchiveHandler.Fullscreen
            PropertyChanges { target: fullscreenImage; opacity: 1 }
            PropertyChanges { target: fullscreenImage; source: previewArchive.currentItem.frontSource }
            StateChangeScript { script: console.log("fullscreen state got started") }
        },

        State {
            name: "fullscreenVideo"
            extend: "visible"
            when: archiveHandler.view == ArchiveHandler.FullscreenVideo
//            PropertyChanges { target: archiveBackground; visible: false }
            PropertyChanges { target: archiveHandler; videoPlaybackActive: true }
//            StateChangeScript { script: videoPlaybackOverlay.forceActiveFocus() }
            StateChangeScript { script: console.log("fullscreenvideo state got started") }
        },
        State {
            name: "infoDialog"
            extend: "imageSelectorView"
            when: archiveHandler.view == ArchiveHandler.Fullscreen
            PropertyChanges { target: archiveInfoDiag; opacity: 1 }
            StateChangeScript { script: console.log("state = infoDialog") }
            StateChangeScript { script: archiveInfoDiag.show() }
        },
        State {
            name: "deleteDialog"
            extend: "imageSelectorView"
            PropertyChanges { target: deleteQuestion; opacity: 1 }
            StateChangeScript { script: console.log("state = deleteDialog") }
            StateChangeScript { script: deleteQuestion.show() }
        },
        State {
            name: "warningMessage"
            extend: "imageSelectorView"
            when: archiveHandler.view == ArchiveHandler.Fullscreen
            PropertyChanges { target: archiveWarningMessage; opacity: 1 }
            StateChangeScript { script: console.log("state = warningMessage") }
            StateChangeScript { script: archiveWarningMessage.show() }
        }
    ]
}
