import QtQuick 1.1
import se.flir 1.0

/**
 * This page shows the main settings area, it essentially represents a single page.
 * Notice that it uses the datamodel to choose between delegate types.
 * See also the SettingsArea and SettingsMixedMenuDelegate
 */

Rectangle {
    id: settingsRoot
    color: "black"
    property variant model: null
    property variant valueModel: null
    property int pageType: Settings.MixedMenuDelegate
    property alias currentIndex: list.currentIndex
    property alias headerText: header.text
    property alias footerText: footer.text
    property alias footerIcon: footer.icon
    property alias showArrows: arrowUp.visible
    signal backClicked
    signal itemTriggered(int item)

    Component {
        id: mixedMenuDelegate
        SettingsMixedMenuDelegate {
            width: settingsRoot.width
        }
    }

    Component {
        id: radioGroupDelegate
        SettingsRadioGroup {
            width: settingsRoot.width
        }
    }

    Component {
        id: radioValuesDelegate
        SettingsRadioValues {
            width: settingsRoot.width
        }
    }

    Component {
        id: radioLanguageDelegate
        SettingsRadioLanguage{
            width: settingsRoot.width
        }
    }

    Component {
        id: valueSelectorDelegate
        SettingsValueSelectorDelegate {
            width: settingsRoot.width
        }
    }

    Component {
        id: nameValueSelectorDelegate
        SettingsNameValueDelegate {
            width: settingsRoot.width
        }
    }

    Component {
        id: fail
        // if you see this delegate, you use the fallback delegate. Which you should not be using ;)
        Item {
            width: settingsRoot.width
            height: text.height * 2
            MouseArea { anchors.fill: parent; onClicked: settingsRoot.itemTriggered(index) }
            Text {
                id: text
                text: qsTrId(modelData.title)
                color: "red"
                anchors.centerIn: parent
            }
        }
    }

    onPageTypeChanged: {
        // our list should have the selected value in the center if its a value list.
        if (pageType == Settings.ValueSelectorDelegate || pageType == Settings.NameValueSelectorDelegate) {
            var cellHeight = (grid.cellHeight + grid.verticalSpacing) * 4 / 5
            list.preferredHighlightBegin = cellHeight
            list.preferredHighlightEnd = cellHeight * 2
            // console.debug(" setting preferredHighlightBegin based on " + list.height
            //         + ", begin: " + list.preferredHighlightBegin
            //         + ", end: " + list.preferredHighlightEnd);
            list.highlightRangeMode = ListView.StrictlyEnforceRange
        } else {
            list.preferredHighlightBegin = 0
            list.preferredHighlightEnd =  0
            list.highlightRangeMode = ListView.NoHighlightRange
        }
    }


    Column {
        anchors.fill: parent
        id: mainColumn
        y: grid.topMargin + grid.verticalSpacing
        property bool isValueList: settingsRoot.model.length == 0

        SettingsHeader {
            id: header
            height: grid.cellHeight + grid.verticalSpacing
            property string text: ""
            setting: qsTrId(text)
            onClicked: settingsRoot.backClicked()

        }

        NavigationArrow {
            id: arrowUp
            anchors.horizontalCenter: parent.horizontalCenter
            onTriggered: settings.selectedIndex = settings.selectedIndex - 1
        }
        Item { width: 1; height: grid.verticalSpacing } // spacer

        ListView {
            id: list
            model: parent.isValueList ? settingsRoot.valueModel : settingsRoot.model
            clip: true
            boundsBehavior: Flickable.StopAtBounds
            property bool needsScrollbar: contentHeight > height
            height: parent.height - header.height
                - grid.verticalSpacing /*this line for the spacing item above */
                - (footer.visible ? footer.height + grid.verticalSpacing : 0)
                - (arrowUp.visible ? arrowUp.height  * 2 : 0)
            focus: true
            anchors.left: parent.left
            anchors.right: parent.right

            delegate: {
                if (settingsRoot.pageType == Settings.MixedMenuDelegate)
                    return mixedMenuDelegate
                if (settingsRoot.pageType == Settings.RadioGroupDelegate ||
                    settingsRoot.pageType == Settings.RadioEmissivityDelegate)
                    return radioGroupDelegate
                if (settingsRoot.pageType == Settings.RadioValuesDelegate)
                    return radioValuesDelegate
                if (settingsRoot.pageType == Settings.ValueSelectorDelegate)
                    return valueSelectorDelegate
                if (settingsRoot.pageType == Settings.NameValueSelectorDelegate)
                    return nameValueSelectorDelegate
                if (settingsRoot.pageType == Settings.HiddenMenuDelegate)
                    return mixedMenuDelegate
                if (settingsRoot.pageType == Settings.RadioLanguageDelegate)
                    return radioLanguageDelegate
                return fail
            }
            highlightMoveDuration: 5
            highlight: Item {
                visible: !arrowUp.visible
                ShadedRectangle {
                    width: grid.width - 4 * grid.horizontalSpacing
                    height: grid.cellHeight - 2 * grid.verticalSpacing
                    anchors.centerIn: parent
                }
            }

            Item { // decorations :)
                anchors.fill: parent
                visible: list.needsScrollbar;

                // Top scroll fader
                Image {
                    id: topScrollCover
                    visible: !arrowUp.visible && list.visibleArea.yPosition > 0
                    source: "../images/Bg_Options_ScrollCoverTop.png"
                    anchors.horizontalCenter: parent.horizontalCenter
                }

                // Bottom scroll fader
                Image {
                    id: bottomScrollCover
                    visible: !arrowUp.visible && list.visibleArea.yPosition < 1 - list.visibleArea.heightRatio - 0.01;
                    source: "../images/Bg_Options_ScrollCoverBottom.png"
                    anchors.bottom: parent.bottom
                    anchors.horizontalCenter: parent.horizontalCenter
                }

                // Scroll bar
                Rectangle {
                    id: scrollBar
                    anchors.right: parent.right
                    anchors.top: parent.top
                    anchors.bottom: parent.bottom
                    width: 3
                    color: "#323232"

                    // Slider
                    Rectangle {
                        color: "#0066cc"
                        anchors.right: parent.right
                        width: parent.width
                        height: list.height * list.visibleArea.heightRatio
                        y: list.visibleArea.yPosition * list.height
                    }
                }
            }

            Keys.onPressed: {
                if (event.key === keys.up) {
                    for (var i = currentIndex - 1; i >= 0; i--) {
                        if (parent.isValueList || list.model[i].enabled) {
                            settings.selectedIndex = i;
                            break;
                        }
                    }
                    event.accepted = true
                }
                else if (event.key === keys.down) {
                    for (var i = currentIndex + 1; i < count; i++) {
                        if (parent.isValueList || list.model[i].enabled) {
                            settings.selectedIndex = i;
                            break;
                        }
                    }
                    event.accepted = true
                }
            }
        }

        NavigationArrow {
            id: arrowDown
            visible: arrowUp.visible
            isUpArrow: false
            anchors.horizontalCenter: parent.horizontalCenter
            onTriggered: {
                var i = list.currentIndex + 1
                if (i < list.count)
                    settings.selectedIndex = i
            }
        }
        Item { width: 1; height: grid.verticalSpacing }

        // Footer
        SettingsFooter {
            id: footer
            anchors.left: parent.left
            anchors.right: parent.right
            height: grid.cellHeight + grid.verticalSpacing + grid.bottomMargin

            visible: {
                if (settingsRoot.pageType == Settings.ValueSelectorDelegate || settingsRoot.pageType == Settings.NameValueSelectorDelegate)
                    return true;
                return false;
            }
        }
    }
    // Text { x: 260; y : 10; color: "yellow"; text: list.count  + (mainColumn.isValueList ? " strings" : " objs")}  // debug :)
}
