import QtQuick 1.1

/**
 MessageBox
 A modal message box, with buttons.

 The message box has two sizes, and depending on content (main-text length essentially)
 it will choose the preferred size.

 Another feature is that you can choose which buttons to show; there are up to 3.
   * okButton which is always left. (Maybe should be renamed to leftButton).
     See property okButtontext, okButtonVisible and the signal okSelected()

   * midButton which is default invisible.
     See property midButtontext, midButtonVisible and the signal midSelected()

   * cancelButton which is always right-most (maybe should be renamed to rightButton)
     see property cancelButtontext, cancelButtonVisible and the signal cancelSelected()
 */


Item {
    id: messageBoxRoot

    // Own properties
    property alias title: titleItem.text
    property alias text: textBoxItem.text
    property alias cancelButtontext: cancelButton.text
    property alias okButtontext: okButton.text
    property alias midButtontext: midButton.text
    property alias okButtonText: okButton.text
    property alias cancelButtonText: cancelButton.text
    property alias tableModel: textTableItem.model
    property bool hasButtons: true
    property bool midButtonVisible: false
    property bool okButtonVisible: true
    property bool cancelButtonVisible: true
    property bool acceptKeyRelease : false
    property bool liveBackground: false

    // not settable properties.
    property bool tableMode: (tableModel === null || tableModel === undefined) ? false : true
    property int numberOfButtons : (midButtonVisible ? 1 : 0) + (okButtonVisible ? 1 : 0) + (cancelButtonVisible ? 1 : 0)

    // Property values
    width: grid.width
    height: grid.height
    opacity: 0
    state: "small"

    // Signals
    signal okSelected
    signal cancelSelected
    signal midSelected

    // Pseudo icon enum
    property string iconStyle: iconEnum.warning
    property alias iconEnum: iconEnum
    QtObject {
        id: iconEnum
        property string warning: "../images/Ic_GlobalDialogue_Warning_Sel.png"
        property string info: "../images/Ic_GlobalDialogue_Info_Sel.png"
        property string question: "../images/Ic_GlobalDialogue_Questionmark_Sel.png"
        property string error: "../images/Ic_GlobalDialogue_Error_Sel.png"
    }

    // Big or small state
    states: [
        State {
            name: "small"
            PropertyChanges { target: messageBox; width: grid.messageBoxWidth }
            PropertyChanges { target: messageBox; height: grid.messageBoxHeight }
        },
        State {
            name: "big"
            PropertyChanges { target: messageBox; width: grid.messageBoxBigWidth }
            PropertyChanges { target: messageBox; height: grid.messageBoxBigHeight }
        }
    ]

    onTableModeChanged: {
        if (tableMode || titleItem.lineCount >= 2 || textBoxItem.lineCount >= 3)
        {
            messageBoxRoot.state = "big"
        }
    }
    onTextChanged: {
        if (tableMode || titleItem.lineCount >= 2 || textBoxItem.lineCount >= 3)
        {
            messageBoxRoot.state = "big"
        }
    }
    onTitleChanged: {
        if (tableMode || titleItem.lineCount >= 2 || textBoxItem.lineCount >= 3)
        {
            messageBoxRoot.state = "big"
        }
    }

    // Debug output
    //onOkSelected: console.log("MessageBox::okSelected()")
    //onCancelSelected: console.log("MessageBox::cancelSelected()")
    //onStateChanged: console.log("MessageBox: State = " + state)

    // Show and hide functions

    function show() {
        //console.log("MessageBox::show()")
        cancelButton.selected = true
        opacity = 1
        forceActiveFocus()
    }

    function hide() {
        //console.log("MessageBox::hide()")
        opacity = 0
        focus = false
        acceptKeyRelease = false
    }

    // Touch handling

    // Cancel buttonless dialog
    MouseArea {
        anchors.fill: parent
        onClicked: {
            if (hasButtons === false)
                cancelSelected()
        }
    }

    Rectangle{
       id: shadow
       visible: !liveBackground
       anchors.fill: parent
       color: "#A0000000"
    }

    // The message box
    BorderImage {
        id: messageBoxImage
        border { left: 12; top: 12; right: 12; bottom: 12 }
        anchors.verticalCenter: parent.verticalCenter
        anchors.horizontalCenter: parent.horizontalCenter
        source: liveBackground ? "../images/Bg_DialoguePopUp_GeneralHardEdges_Def.png" : "../images/Bg_DialoguePopUp_General_Def.png"

        // Make /messageBox/ fit exactly whithin the visual black rectangle (the messageBoxImage images have transparent margins of diferent size)
        width: messageBox.width + 12
        height: messageBox.height + (liveBackground ? 12 : 13)


        Item {
            id: messageBox
            // Make /messageBox/ fit exactly whithin the visual black rectangle (the images have transparent margins of diferemt size)
            x: 6
            y: liveBackground ? 6 : 7

            // The icon
            Image {
                id: icon
                anchors.verticalCenter: titleItem.verticalCenter
                anchors.left: parent.left
                anchors.leftMargin: grid.horizontalSpacing * 2
                source: iconStyle
            }

            // The title text
            Text {
                id: titleItem
                anchors.top: parent.top
                anchors.topMargin: grid.verticalSpacing * 3
                anchors.left: icon.right
                anchors.leftMargin: grid.horizontalSpacing * 2
                width: parent.width - icon.width - grid.horizontalSpacing * 4
                font.family: fonts.family
                font.pixelSize: fonts.smallSize
                font.bold: true
                color: colors.textFocused
                wrapMode: Text.Wrap
                maximumLineCount: 3
            }

            // The blue line

            BorderImage {
                id: blueLine
                anchors.top: titleItem.bottom
                anchors.topMargin: titleItem.lineCount === 1 ? grid.verticalSpacing * 3 : grid.verticalSpacing * 2
                anchors.left: parent.left
                anchors.leftMargin: grid.horizontalSpacing
                anchors.right: parent.right
                anchors.rightMargin: grid.horizontalSpacing
                border { left: 50; top: 0; right: 50; bottom: 0 }
                source: "../images/Bg_DialoguePopUp_Line_Def.png"
            }

            // The message

            Item {
                id: textItem
                //property alias text: textBoxItem.text
                anchors.top: blueLine.bottom
                anchors.topMargin: grid.verticalSpacing * 5
                anchors.left: parent.left
                anchors.leftMargin: grid.horizontalSpacing * 7
                anchors.right: parent.right
                anchors.rightMargin: grid.horizontalSpacing * 7
                anchors.bottom: parent.bottom
                anchors.bottomMargin: grid.verticalSpacing * 5
                clip: true
                property bool showScrollbar: textTableItem.contentHeight > textTableItem.height ? true : false
                property int scrollHeight: textTableItem.height * textTableItem.visibleArea.heightRatio
                property int scrollPos: textTableItem.visibleArea.yPosition * textTableItem.height
                property alias contentY: textTableItem.contentY
                property alias contentHeight: textTableItem.contentHeight

                // Show text in text box
                Text {
                    id: textBoxItem
                    visible: tableMode === false
                    anchors.fill: parent
                    maximumLineCount: 6
                    wrapMode: Text.WordWrap
                    font.family: fonts.family
                    font.pixelSize: fonts.smallSize
                    color: colors.textFocused
                }

                // or: Show text from model in a table (see archive info)
                ListView {
                    id: textTableItem
                    visible: tableMode
                    anchors.fill: parent
                    spacing: grid.verticalSpacing
                    flickableDirection: Flickable.VerticalFlick
                    boundsBehavior: Flickable.StopAtBounds

                    delegate: Item {
                        id: delegateId
                        width: parent.width
                        height: delegateLabel.height
                        Text {
                            id: delegateLabel
                            font.family: fonts.family
                            font.pixelSize: fonts.smallSize
                            color: colors.textFocused
                            anchors.left: parent.left
                            text: label
                        }
                        Text {
                            id: delegateValue
                            font.family: fonts.family
                            font.pixelSize: fonts.smallSize
                            color: colors.textFocused
                            anchors.right: parent.right
                            text: value
                        }
                    }
                }
            }

            // Scroll bar
            Item {
                id: scrollBar
                visible: tableMode && textItem.showScrollbar
                anchors.right: parent.right
                anchors.rightMargin: width + 3
                anchors.top: textItem.top
                anchors.bottom: textItem.bottom
                width: 3

                // Background
                Rectangle {
                    color: "#323232"
                    anchors.fill: parent
                }

                // Slider
                Rectangle {
                    color: "#0066cc"
                    anchors.right: parent.right
                    width: parent.width
                    height: textItem.scrollHeight
                    y: textItem.scrollPos
                }
            }

            // The buttons

            Row {
                id: buttonRow
                objectName: "messageButtonRow"
//                property int buttonWidth : (buttonRow.width -grid.horizontalSpacing * 2 ) / messageBoxRoot.numberOfButtons - 1*messageBoxRoot.numberOfButtons
                property int buttonWidth : (buttonRow.width) / messageBoxRoot.numberOfButtons - 4

                spacing: grid.horizontalSpacing * 2

                anchors.left: parent.left
                anchors.leftMargin: grid.horizontalSpacing * 1
                anchors.right: parent.right
                anchors.rightMargin: grid.horizontalSpacing * 1
                anchors.bottom: parent.bottom
                anchors.bottomMargin: grid.verticalSpacing * 1

                Button {
                    id: okButton
                    text: "YES"
                    visible: hasButtons && okButtonVisible
                    width:  buttonRow.buttonWidth
                    selected: false

                    onSelectedChanged: {
                        if (selected) {
                            midButton.selected = false
                            cancelButton.selected = false
                        }
                    }
                    onClicked: okSelected()
                }

                Button {
                    id: midButton
                    text: "MAYBE" // Never shown...
                    visible: hasButtons && midButtonVisible
                    width:  buttonRow.buttonWidth

                    selected: false
                    onSelectedChanged: {
                        if (selected) {
                            okButton.selected = false
                            cancelButton.selected = false
                        }
                    }
                    onClicked: midSelected()
                }

                Button {
                    id: cancelButton
                    text: "NO"
                    visible: hasButtons && cancelButtonVisible
                    width:  buttonRow.buttonWidth

                    selected: true
                    onSelectedChanged: {
                        if (selected) {
                            okButton.selected = false
                            midButton.selected = false
                        }
                    }
                    onClicked: cancelSelected()
                }
            }
        }
    }

    // Keyboard handling
    Keys.onPressed: {
        if (event.isAutoRepeat && event.key === keys.trigger)
        {
            event.accepted = true
            return
        }

        acceptKeyRelease = true

        if (event.key !== keys.onOff)
            event.accepted = true
    }

    Keys.onReleased: {
        if (acceptKeyRelease)
        {
            if (hasButtons === false)
            {
                if (textItem.showScrollbar)
                {
                    if (event.key === keys.up)
                    {
                        textItem.contentY -= 20
                        if (textItem.contentY < 0)
                            textItem.contentY = 0
                    }
                    else if (event.key === keys.down)
                    {
                        textItem.contentY += 20
                        if (textItem.contentY + textItem.height > textItem.contentHeight)
                            textItem.contentY = textItem.contentHeight - textItem.height
                    }
                    else if(!event.isAutoRepeat)
                        cancelSelected()
                }
                else if(!event.isAutoRepeat)
                    cancelSelected()
                event.accepted = true
            }
            else
            {
                if (event.key === keys.left) {
                    if (cancelButton.selected)
                    {
                        if (midButton.visible)
                            midButton.selected = true
                        else
                            okButton.selected = true
                    } else if (midButton.selected)
                        okButton.selected = true
                    event.accepted = true
                }
                else if (event.key === keys.right) {
                    if (okButton.selected)
                    {
                        if (midButton.visible)
                            midButton.selected = true
                        else
                            cancelButton.selected = true
                    } else if (midButton.selected)
                        cancelButton.selected = true
                    event.accepted = true
                }
                else if (event.key === keys.select) {
                    if (okButton.selected)
                        okSelected()
                    else if (midButton.selected)
                        midSelected()
                    else
                        cancelSelected()
                    event.accepted = true
                }
                else if (event.key === keys.back || event.key === keys.archive) {
                    cancelSelected()
                    event.accepted = true
                }
                else if (event.key === keys.up || event.key === keys.down) {
                    // Prevent leaking keys to parent
                    event.accepted = true
                }
            }
        }
    }
}
