import QtQuick 1.1

// Soft keyboard evaluation
import "SoftKeyboard"

FocusScope {
    property variant model
    signal focusDesktopFocusGroup

    id: softKeyboardPage

    x: 0
    y: grid.height
    width: grid.width
    height: grid.height

    visible: false
    state: "hide"

    property double topOpacity: 1
    Rectangle {
        color: colors.background
        anchors.bottom: parent.top
        anchors.left: parent.left
        anchors.right: parent.right
        height: parent.height
        opacity: parent.topOpacity
    }


    states: [
        State {
            name: "show"
            PropertyChanges {
                target: softKeyboardPage
                y: 0
                visible: true
                topOpacity: 1
            }
        }
        ,
        State {
            name: "hide"
            PropertyChanges {
                target: softKeyboardPage
                y: grid.height
                visible: false
                topOpacity: 0
            }
        }
        ,
        State {
            when: (softKeyboardPage.model.row0TotalWeight > 0)
            name: "modelAssigned"
            StateChangeScript {
                 name: "Init"
                 script: keyboard.init();
             }
        }
    ]

    transitions: [
        Transition {
            to: "hide"
            SequentialAnimation {
                // Slide out.
                PropertyAnimation {
                    properties: "y,topOpacity"
                    duration: 200
                    easing.type: Easing.InOutQuad
                }
                // Change visibility.
                PropertyAction {
                    property: "visible"
                }
            }
        },
        Transition {
            to: "show"
            SequentialAnimation {
                // Change visibility.
                PropertyAction {
                    property: "visible"
                }
                // Slide in.
                PropertyAnimation {
                    properties: "y,topOpacity"
                    duration: 200
                    easing.type: Easing.InOutQuad
                }
            }
        }
    ]


    // Functions used to show and hide pages
    function show() {
        if (state === "show") {
            return
        }
        state = "show"
        greenbox.system.blockAutoShutdown()
        textEditBox.forceActiveFocus()
    }

    function hide() {
        if (state === "hide") {
            return
        }
        state = "hide"
        model.keyboardShown = false
        greenbox.system.allowAutoShutdown()
        focusDesktopFocusGroup()
    }


    // Key handling
    Keys.onPressed: {
        if (event.key === keys.onOff) {
            hide()
            event.accepted = true
        }
    }

    Keys.onReleased: {
        if (event.key === keys.back || event.key === keys.archive || event.key === keys.power) {
            hide()
            event.accepted = true
        }
    }


    // The page itself, and its contents
    Rectangle {
        color: colors.background
        anchors.fill: parent

        MouseArea { // eat any stray mouse clicks.
            anchors.fill: parent
        }

        // Header
        SettingsHeader {
            id: settingsHeader
            setting: qsTrId(model.header) + translation.update
            onClicked: hide()
        }

        property bool keyboardShown: model.keyboardShown
        onKeyboardShownChanged: {
            if (model.keyboardShown === true) {
                parent.show()
            }
            else {
                parent.hide()
            }
        }

        Item {
            id: flickableBase

            anchors.left: parent.left
            anchors.right: parent.right
            anchors.rightMargin: 3
            anchors.leftMargin: 3
            anchors.top: settingsHeader.bottom
            anchors.bottom: keyboard.top

            Image{
                id: textBoxBackground
                source:  "../images/Kb_Bg_TextBox_Def.png"
                anchors.margins: 5
                anchors.top: parent.top
                anchors.left: parent.left
                // anchors.bottom: parent.bottom
                anchors.right: scrollBar.left

                Image{
                    id: removeButton
                    source:  "../images/Kb_Bg_TextBox_Clear_Def.png"
                    anchors.margins: grid.horizontalSpacing*2
                    anchors.top: parent.top
                    anchors.right: parent.right

                    MouseArea{
                        anchors.fill: parent
                        onClicked: keyboard.removeAll()
                    }
                }

                Flickable {
                    id: textEditFlickable
                    anchors.topMargin: 4
                    anchors.bottomMargin: 4
                    anchors.leftMargin: 6
                    anchors.rightMargin: 6
                    anchors.top: parent.top
                    anchors.left: parent.left
                    anchors.bottom: parent.bottom
                    anchors.right: removeButton.left

                    contentWidth: textEditFlickable.width
                    contentHeight: textEditBox.height

                    flickableDirection: Flickable.HorizontalAndVerticalFlick
                    boundsBehavior: Flickable.StopAtBounds

                    clip: true

                    function ensureVisible(r) {
                        if (contentY >= r.y)
                            contentY = r.y;
                        else if (contentY + height <= r.y + r.height)
                            contentY = r.y + r.height - height;
                    }


                    TextEdit {
                        id: textEditBox
                        wrapMode: TextEdit.Wrap
                        width: parent.width
                        textFormat: TextEdit.PlainText
                        text: model.textValue
                        onTextChanged: {
                            model.textValue = text
                            cursorPosition = text.length
                        }
                        readOnly: false
                        focus: true

                        color: keyboard.primaryCharacterColor
                        font.pixelSize: keyboard.primaryCharacterSize

                        onCursorRectangleChanged: textEditFlickable.ensureVisible(cursorRectangle)
                    }
                }
            }

            // Scroll bar
            Item {
                id: scrollBar
                visible: textEditBox.height > flickableBase.height
                anchors.right: parent.right
                anchors.top: textBoxBackground.top
                anchors.bottom: textBoxBackground.bottom
                width: 3
                clip: true

                // Background
                Rectangle {
                    id:scrollBarBackground
                    color: "#323232"
                    anchors.fill: parent
                }

                // Foreground
                Rectangle {
                    color: "#0066cc"
                    anchors.right: parent.right
                    width: parent.width
                    height: Math.pow(flickableBase.height, 2) / textEditBox.height
                    y: textEditFlickable.visibleArea.yPosition * (flickableBase.height - height)

                }
            }
        }

        Keyboard {
            id: keyboard
            anchors.fill: parent
            anchors.topMargin: parent.height * 0.4
            anchors.rightMargin: grid.horizontalSpacing
            anchors.leftMargin: grid.horizontalSpacing
            anchors.bottomMargin: grid.verticalSpacing + grid.bottomMargin
            model: softKeyboardPage.model

            Binding {
                target: keyboard
                property: "textField"
                value: textEditBox
            }
        }
    }
}
